@@ -30,18 +30,7 @@
 G_BEGIN_DECLS
 
 #define CLOCK_ICON "gnome-panel-clock"
-
-/* Needs to match the indices in the combo of the prefs dialog */
-typedef enum {
-	CLOCK_FORMAT_INVALID = 0,
-	CLOCK_FORMAT_12,
-	CLOCK_FORMAT_24,
-	CLOCK_FORMAT_UNIX,
-	CLOCK_FORMAT_INTERNET,
-	CLOCK_FORMAT_CUSTOM
-} ClockFormat;
-
-ClockFormat clock_locale_format (void);
+#define CLOCK_SCHEMA_DIR "/schemas/apps/clock_applet/prefs"
 
 G_END_DECLS
 
